// -------------------------------------------
// Chip II - Robotics Experimentation Platform
// -------------------------------------------

// Header files
#include <hc11e1.h>  // 68HC11E1
#include <stdio.h>   // Standard I/O
#include <string.h>  // String handling
#include "utils.h"   // General utility library
#include "display.h" // LCD display
#include "keypad.h"  // Keypad
#include "power.h"   // Battery power gauge
#include "drive.h"   // Drive motor control
#include "ir.h"      // IR detector
#include "temp.h"    // Temperature sensor
#include "sonar.h"   // Sonar ranging sensor
#include "speech.h"  // V8600 Speech Synth
#include "exp.h"     // I/O Expansion ports


void demoNavFunc(int l_rng, int c_rng, int r_rng)
{
 char str[14];
 int odoL,odoR;

 odoL=driveOdometerL();
 odoR=driveOdometerR();

 // average l/r range if similar
 if((c_rng>50)&&(l_rng>20)&&(l_rng>20)&&(l_rng<r_rng)&&((r_rng-l_rng)<6))
 {
  l_rng=(l_rng+r_rng)/2;
  r_rng=(l_rng+r_rng)/2;
 }
 else if((c_rng>50)&&(l_rng>20)&&(l_rng>20)&&(r_rng<l_rng)&&((l_rng-r_rng)<6))
 {
  l_rng=(l_rng+r_rng)/2;
  r_rng=(l_rng+r_rng)/2;
 }
 
 irScan();

    // distant obstacle, all vectors
    if((c_rng>=100)&&(l_rng>=50)&&(r_rng>=50)
     &&(!irStatus(CENTER))&&(!irStatus(LEFT))&&(!irStatus(RIGHT)))
    {
     driveControl(LEFT,FWD,70);
     driveControl(RIGHT,FWD,70);
     speechStr("Look at all this space!");
    }

    // immediate danger
    else if((irStatus(CENTER))||(c_rng<8))
    {
     driveControl(LEFT,driveDir(LEFT),0);
     driveControl(RIGHT,driveDir(RIGHT),0);
     delay(200);
     driveControl(LEFT,REV,50);
     driveControl(RIGHT,REV,50);
     servoPos(LEFT,55);
     servoPos(RIGHT,45);
     speechStr("Oops!");
     while((((sonarRange(LEFT)+sonarRange(RIGHT))/2)<15)||(irStatus(CENTER)))
     {
      irScan();
      if(irStatus(REAR))
      {
       driveControl(LEFT,driveDir(LEFT),0);
       driveControl(RIGHT,driveDir(RIGHT),0);
       delay(200);
       break;
      }
     }
    }

    // L=very near (IR)
    else if(irStatus(LEFT))
    {
     // stop moving
     driveControl(LEFT,driveDir(LEFT),0);
     driveControl(RIGHT,driveDir(RIGHT),0);
     speechStr("Oops!");
     delay(200);

     // back away from obstacle
     driveControl(LEFT,REV,50);
     driveControl(RIGHT,REV,50);
     while((!irStatus(REAR))&&(irStatus(LEFT))) { irScan(); }

     // turn away from obstacle
     driveControl(LEFT,REV,30);
     while((!irStatus(REAR))&&(driveOdometerR()<(odoR+15))) { irScan(); }

     // stop and await new loop cycle
     driveControl(LEFT,driveDir(LEFT),0);
     driveControl(RIGHT,driveDir(RIGHT),0);
     delay(200);
    }

    // R=very near (IR)
    else if(irStatus(RIGHT))
    {
     // stop moving
     driveControl(LEFT,driveDir(LEFT),0);
     driveControl(RIGHT,driveDir(RIGHT),0);
     speechStr("Oops!");
     delay(200);

     // back away from obstacle
     driveControl(LEFT,REV,50);
     driveControl(RIGHT,REV,50);
     while((!irStatus(REAR))&&(irStatus(RIGHT))) { irScan(); }

     // turn away from obstacle
     driveControl(RIGHT,REV,30);
     while((!irStatus(REAR))&&(driveOdometerL()<(odoL+15))) { irScan(); }

     // stop and await new loop cycle
     driveControl(LEFT,driveDir(LEFT),0);
     driveControl(RIGHT,driveDir(RIGHT),0);
     delay(200);
    }

    // C=very near L>R (sonar)
    else if((c_rng<11)&&(l_rng>r_rng))
    {
     driveControl(LEFT,driveDir(LEFT),0);
     driveControl(RIGHT,driveDir(RIGHT),0);
     delay(200);
     driveControl(LEFT,REV,30);
     driveControl(RIGHT,FWD,30);
     servoPos(RIGHT,70);
     speechStr("Oops!");
     while(sonarRange(RIGHT)<(r_rng+11))
     {
      // protection from crazy spinning
      if((driveOdometerL()>(odoL+10))||(driveOdometerR()>(odoR+10))||(irStatus(ANY)))
      {
       irScan();
       break;
      }
     }
    }
    // C=very near L<R (sonar)
    else if((c_rng<11)&&(l_rng<r_rng))
    {
     driveControl(LEFT,driveDir(LEFT),0);
     driveControl(RIGHT,driveDir(RIGHT),0);
     delay(200);
     driveControl(LEFT,FWD,30);
     driveControl(RIGHT,REV,30);
     servoPos(LEFT,30);
     speechStr("Oops!");
     while(sonarRange(LEFT)<(l_rng+5))
     {
      // protection from crazy spinning
      irScan();
      if((driveOdometerL()>(odoL+10))||(driveOdometerR()>(odoR+10))||(irStatus(ANY)))
      {
       irScan();
       break;
      }
     }

    }

    // C=distant L=near R=near
    else if((c_rng>=100)&&(l_rng<50)&&(r_rng<50))
    {
     driveControl(LEFT,FWD,40);
     driveControl(RIGHT,FWD,40);
    }

    // C=distant L=near R=distant
    else if((c_rng>=100)&&(l_rng<50)&&(r_rng>=50))
    {
     driveControl(LEFT,FWD,50);
     driveControl(RIGHT,FWD,40);
    }
    // C=distant L=distant R=near
    else if((c_rng>=100)&&(l_rng>=50)&&(r_rng<50))
    {
     driveControl(LEFT,FWD,40);
     driveControl(RIGHT,FWD,50);
    }

    // C=near L=near R=distant
    else if((c_rng<100)&&(l_rng<50)&&(r_rng>=50))
    {
     driveControl(LEFT,FWD,40);
     driveControl(RIGHT,FWD,30);
    }
    // C=near L=distant R=near
    else if((c_rng<100)&&(l_rng>=50)&&(r_rng<50))
    {
     driveControl(LEFT,FWD,30);
     driveControl(RIGHT,FWD,40);
    }

    // C=near L>R
    else if((c_rng<100)&&(l_rng>r_rng))
    {
     driveControl(LEFT,FWD,30);
     driveControl(RIGHT,FWD,35);
    }
    // C=near L<R
    else if((c_rng<100)&&(l_rng<r_rng))
    {
     driveControl(LEFT,FWD,35);
     driveControl(RIGHT,FWD,30);
    }

    else if((c_rng<100)&&(l_rng==r_rng))
    {
     driveControl(LEFT,FWD,40);
     driveControl(RIGHT,FWD,40);
    }

    // Unknown condition
    else
    {
     sprintf(str,"  %03d %03d %03d",l_rng,c_rng,r_rng);
     dispPutStrXY(lcdLg1,1,2,str);
     driveControl(LEFT,FWD,0);
     driveControl(RIGHT,FWD,0);
     delay(500);
    }

}

void demoNav(void)
{
 char ch;
 int tmp1,tmp2;
 int l_rng,c_rng,r_rng;
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1,"  Nav Demo");
 dispPutStrXY(lcdLg1,1,2,"          ");
 ch=kpadGetchar();
 while(ch!='#')
 {
  // Autonomous sweep and drive
  if(ch=='5')
  {
   speechStr("Navigating.");
   l_rng=0;
   c_rng=0;
   r_rng=0;
   while(1==1)
   {
    // Left
    servoPos(LEFT,30);
     l_rng=sonarRange(LEFT);

    // Center
    servoPos(LEFT,55);
     tmp1=sonarRange(LEFT);
    servoPos(RIGHT,45);
     tmp2=sonarRange(RIGHT);
    c_rng=(tmp1+tmp2)/2;
     
    // Right
    servoPos(RIGHT,70);
     r_rng=sonarRange(RIGHT);

    demoNavFunc(l_rng,c_rng,r_rng);
    delayms(50);

    // Right
    servoPos(RIGHT,70);
     r_rng=sonarRange(RIGHT);

    // Center
    servoPos(LEFT,55);
     tmp1=sonarRange(LEFT);
    servoPos(RIGHT,45);
     tmp2=sonarRange(RIGHT);
    c_rng=(tmp1+tmp2)/2;

    // Left
    servoPos(LEFT,30);
     l_rng=sonarRange(LEFT);

    demoNavFunc(l_rng,c_rng,r_rng);
    delayms(50);

   }  
  }
  ch=kpadGetchar();
 }
}

void demoSonar(void)
{
 char ch, str[12];
 int rng;
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1,"  Sonar Demo");
 dispPutStrXY(lcdLg1,1,2,"            ");

 bitSet(&_H11PORTB,6,OFF); // clear pulse

 ch=kpadGetchar();
 while(ch!='#')
 {
  if(ch=='1')
  {
   rng=sonarRange(LEFT);
   if(rng>0)
   {
    sprintf(str,"  %d  ",rng);
    dispPutStrXY(lcdLg1,1,2,"            ");
    dispPutStrXY(lcdLg1,1,2,str);
   }
  }
  if(ch=='3')
  {
   rng=sonarRange(RIGHT);
   if(rng>0)
   {
    sprintf(str,"  %d  ",rng);
    dispPutStrXY(lcdLg1,1,2,"            ");
    dispPutStrXY(lcdLg1,1,2,str);
   }
  }
  if(ch=='7')
  {
   while(1)
   {
    rng=sonarRange(LEFT);
    if(rng>0)
    {
     sprintf(str,"  %02d inches",rng);
     dispPutStrXY(lcdLg1,1,2,str);
    }
   }
  }
  if(ch=='9')
  {
   while(1)
   {
    rng=sonarRange(RIGHT);
    if(rng>0)
    {
     sprintf(str,"  %02d inches",rng);
     dispPutStrXY(lcdLg1,1,2,str);
    }
   }
  }
  ch=kpadGetchar();
 }
}


void demoPWM(void)
{
 char ch,str[14];
 int lspd,rspd,ldir,rdir;
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1,"   PWM Demo ");
 lspd=0;
 rspd=0;
 ldir=FWD;
 rdir=FWD;
 sprintf(str,"  %03d%%  %03d%%",lspd,rspd);
 dispPutStrXY(lcdLg1,1,2,str);

 ch=kpadGetchar();
 while(ch!='#')
 {
  // left
  if(ch=='1') lspd+=10;
  if(ch=='4') lspd=0;
  if(ch=='7') lspd-=10;

  // right
  if(ch=='3') rspd+=10;
  if(ch=='6') rspd=0;
  if(ch=='9') rspd-=10;

  // both
  if(ch=='2') { lspd+=10; rspd+=10; }
  if(ch=='5') { lspd=0; rspd=0; }
  if(ch=='8') { lspd-=10; rspd-=10; }

  // direction
  if(ch=='A') { ldir=FWD; }
  if(ch=='B') { ldir=REV; }
  if(ch=='C') { rdir=FWD; }
  if(ch=='D') { rdir=REV; }

  intMinMax(&lspd,0,100);
  intMinMax(&rspd,0,100);
  driveControl(LEFT,ldir,lspd);
  driveControl(RIGHT,rdir,rspd);

  sprintf(str,"  %03d%%  %03d%%",lspd,rspd);
  dispPutStrXY(lcdLg1,1,2,str);

  ch=kpadGetchar();
 }
}

void demoIR(void)
{
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1,"   IR Demo");
 dispPutStrXY(lcdLg1,1,2,"          ");

 while(1)
 {
  irScan();

  if(irStatus(CENTER)==ON)
   dispPutStrXY(lcdLg1,10,2,"CENTER");
  else
   dispPutStrXY(lcdLg1,10,2,"      ");

  if(irStatus(LEFT)==ON)
   dispPutStrXY(lcdLg1,1,2,"LEFT  ");
  else
   dispPutStrXY(lcdLg1,1,2,"      ");

  if(irStatus(RIGHT)==ON)
   dispPutStrXY(lcdLg1,20,2,"RIGHT ");
  else
   dispPutStrXY(lcdLg1,20,2,"      ");

  if(irStatus(REAR)==ON)
   dispPutStrXY(lcdLg2,10,1,"REAR  ");
  else
   dispPutStrXY(lcdLg2,10,1,"      ");
 }
}

void demoTemp(void)
{
 char str[10];
 int temp;
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1," Temperature");

 temp=tempReading();
 sprintf(str,"  %d degrees",temp);
 dispPutStrXY(lcdLg1,1,2,str);
 speechStr(str);

 while(1)
 {
  temp=tempReading();
  sprintf(str,"  %d degrees",temp);
  dispPutStrXY(lcdLg1,1,2,str);
  delay(5);
 }
}

void demoPower(void)
{
 int i,level;
 char str[14];
 level=powerLevel();
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1,"E[");
 for(i=1;i<=level;i++)
  dispPutCharXY(lcdLg1,2+i,1,'*');
 dispPutStrXY(lcdLg1,13,1,"]F");
 sprintf(str,"  %.01f volts",powerVolts());
 dispPutStrXY(lcdLg2,1,1,str);
 speechStr(str);
 kpadGetchar();
}

void demoGear(void)
{
 char str[12];
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1," Gear Sensor");

 while(1)
 {
  sprintf(str,"%d",driveOdometerL());
  dispPutStrXY(lcdLg1,1,2,str);
  sprintf(str,"%d",driveOdometerR());
  dispPutStrXY(lcdLg1,7,2,str);

  sprintf(str,"%d",driveVel(LEFT));
  dispPutStrXY(lcdLg2,1,2,str);
  sprintf(str,"%d",driveVel(RIGHT));
  dispPutStrXY(lcdLg2,7,2,str);
 }
}

void demoSpeech(void)
{
 char ch;
 int i;

 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1,"  Speech Demo");
 dispPutStrXY(lcdLg1,1,2,"  -----------");

 ch=kpadGetchar();
 while(ch!='#')
 {
  if(ch=='1') speechStr("Hello, my name is Chip 2!");
  if(ch=='2') speechStr("Danger!  Danger Will Robinson!");
  if(ch=='3') speechStr("Hey Nimby.  Meyaow, meyaow!");
  if(ch=='5')
  {
   speechStr("We are Chip of Borg!  Resistance is futile!  You will be assimilated!  We will add your biological and technological distinctiveness to our own!  Your culture will adapt to service ours!");
   for(i=0;i<=8;i++)
   {
    servoPos(LEFT,25); servoPos(RIGHT,0);
    delay(200);
    servoPos(LEFT,0);  servoPos(RIGHT,50);
    delay(200);
    servoPos(LEFT,50);  servoPos(RIGHT,75);
    delay(200);
    servoPos(LEFT,75);  servoPos(RIGHT,25);
    delay(200);
    servoPos(LEFT,50);  servoPos(RIGHT,50);
    delay(200);
   }
  }
  if(ch=='0') speechStr("No one can be told what the matrix is.");
  ch=kpadGetchar();
 }
}

void demoFeedback(void)
{
 char ch;
 int rng;
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1,"  Feedback Demo");
 dispPutStrXY(lcdLg1,1,2,"  -------------");
 dispPutStrXY(lcdLg2,1,1,"  1) Sonar sweep    2) IR Sweep");
 dispPutStrXY(lcdLg2,1,2,"  3) Temperature    4) Battery voltage");
 ch=kpadGetchar();
 while(ch!='#')
 {
  if(ch=='1')
  {
   rng=0;
   servoPos(LEFT,0);
   while(getServoPos(LEFT)<100)
   {
    rng=sonarRange(LEFT);
    printf("[S/L=%i,%i]\n",getServoPos(LEFT),rng);
    servoPos(LEFT,getServoPos(LEFT)+5);
   }
   servoPos(LEFT,50);
  }
  if(ch=='2')
  {
   irScan();
   printf("[IR/C=%i]\n",irStatus(CENTER));
   printf("[IR/L=%i]\n",irStatus(LEFT));
   printf("[IR/R=%i]\n",irStatus(RIGHT));
   printf("[IR/B=%i]\n",irStatus(REAR));
  }
  if(ch=='3')
  {
   printf("[TEMP/*=%d]\n",tempReading());
  }
  if(ch=='4')
  {
   printf("[B/1=%.01f]\n",powerVolts());
  }
  ch=kpadGetchar();
 }
}

void demoServo(void)
{
 char ch;
 int servo;
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1,"  Servo Demo");
 dispPutStrXY(lcdLg1,1,2,"  ----------");
 servo=LEFT;
 servoPos(servo,50);
 ch=kpadGetchar();
 while(ch!='#')
 {
  if(ch=='1') servoPos(servo,0);
  if(ch=='2') servoPos(servo,50);
  if(ch=='3') servoPos(servo,100);
  if(ch=='4') servoPos(servo,25);
  if(ch=='5') servoPos(servo,50);
  if(ch=='6') servoPos(servo,75);
  if(ch=='7') servo=LEFT;
  if(ch=='9') servo=RIGHT;
  ch=kpadGetchar();
 }
}

void demoCompass(void)
{
 char str[14];
 int value;
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1,"  Compass Demo");
 dispPutStrXY(lcdLg1,1,2,"  ------------");

 expOut(EXP_OUT0,3,1); // select
 _H11DDRD=0x3A;
 _H11SPCR=0x5F; // CPOL=1, CPHA=1, E/32
 value=_H11SPSR;
 value=_H11SPDR;
 expOut(EXP_OUT0,2,1); // P/C
 while(1)
 {
  expOut(EXP_OUT0,2,0); // P/C
  delayms(10);
  expOut(EXP_OUT0,2,1); // P/C
  while(!(_H11PORTE & bit(6)));
  delayms(10);
  expOut(EXP_OUT0,3,0); // select
  delayms(5);
  _H11SPDR=0;
  while(!(_H11SPSR & 0x80));
  value=_H11SPDR*256;
  _H11SPDR=0;
  while(!(_H11SPSR & 0x80));
  value=value+_H11SPDR;
  expOut(EXP_OUT0,3,1); // select

// alter reading to match mounting direction  
  value+=90;
  if(value>359) value-=360;

// print results to LCD
  sprintf(str,"  %03d  ",value);
  dispPutStrXY(lcdLg2,1,1,str);
  delay(250);  
 }
}

void main()
{
 char ch;

 // Initialize drive system
 driveInit();

 // Initialize IR proximity sensors
 irInit();

 // General initialization
 initGeneral();

 dispInit();

 speechCmd(VOICE,VOICE_Skip);
 speechCmd(EXPRESSION,8);
 speechCmd(TEXT_MODE,2);

 dispPutIns(lcdLg1,Clear_Display);
 dispPutIns(lcdLg2,Clear_Display);
 dispPutStrXY(lcdLg1,11,1,"-=*=- Chip 2 -=*=-");

 ch=kpadGetchar();
 while(ch!='*')
 {
  if(ch=='A') demoNav();
  if(ch=='B') demoPWM();
  if(ch=='C') demoIR();
  if(ch=='D') demoSonar();
  if(ch=='1') demoTemp();
  if(ch=='2') demoPower();
  if(ch=='3') demoSpeech();
  if(ch=='4') demoFeedback();
  if(ch=='5') demoCompass();
  if(ch=='7') demoGear();
  if(ch=='9') demoServo();
  dispPutIns(lcdLg1,Clear_Display);
  dispPutIns(lcdLg2,Clear_Display);
  dispPutStrXY(lcdLg1,11,1,"-=*=- Chip 2 -=*=-");
  ch=kpadGetchar();
 }
 haltGeneral();
 dispPutIns(lcdLg1,Clear_Display);
 dispPutStrXY(lcdLg1,1,1,"Program halted!");
}

